@@ -1,6 +1,6 @@
 /********************************************
 field.h
-copyright 2009-2020,2023 Thomas E. Dickey
+copyright 2009-2023,2024 Thomas E. Dickey
 copyright 1991-1995,2014 Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,7 +11,7 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: field.h,v 1.15 2023/07/23 11:33:56 tom Exp $
+ * $MawkId: field.h,v 1.20 2024/09/05 17:44:48 tom Exp $
  */
 
 /* field.h */
@@ -19,8 +19,8 @@ the GNU General Public License, version 2, 1991.
 #ifndef  MAWK_FIELD_H
 #define  MAWK_FIELD_H   1
 
-#include "nstd.h"
-#include "types.h"
+#include <nstd.h>
+#include <types.h>
 
 extern void set_field0(const char *, size_t);
 extern void split_field0(void);
@@ -28,7 +28,7 @@ extern void field_assign(CELL *, CELL *);
 extern char *is_string_split(PTR, size_t *);
 extern void slow_cell_assign(CELL *, CELL *);
 extern CELL *slow_field_ptr(int);
-extern int field_addr_to_index(CELL *);
+extern int field_addr_to_index(const CELL *);
 extern void set_binmode(long);
 
 #define  NUM_PFIELDS		5
@@ -75,22 +75,20 @@ extern int nf;			/* shadows NF */
 #define  SEP_RE         3
 #define  SEP_MLR	4
 
-typedef struct {
+typedef struct _separator
+#ifdef Visible_SEPARATOR
+{
     char type;
     char c;
-    PTR ptr;			/* STRING* or RE machine* */
-} SEPARATOR;
+    union {
+	STRING *s_ptr;
+	RE_NODE *r_ptr;
+    } u;
+}
+#endif
+SEPARATOR;
 
 extern SEPARATOR rs_shadow;
 extern CELL fs_shadow;
 
-/*  types for splitting overflow */
-
-typedef struct spov {
-    struct spov *link;
-    STRING *sval;
-} SPLIT_OV;
-
-extern SPLIT_OV *split_ov_list;
-
 #endif /* MAWK_FIELD_H  */
