@@ -1,6 +1,6 @@
 /********************************************
 fin.c
-copyright 2008-2021,2023, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 1991-1995,1996, Michael D. Brennan
 
 This is a source file for mawk, an implementation of
@@ -11,18 +11,22 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: fin.c,v 1.52 2023/10/31 21:21:24 tom Exp $
+ * $MawkId: fin.c,v 1.57 2024/09/05 17:38:30 tom Exp $
  */
 
-/* fin.c */
+#define Visible_CELL
+#define Visible_FIN
+#define Visible_SEPARATOR
+#define Visible_STRING
+#define Visible_SYMTAB
 
-#include "mawk.h"
-#include "fin.h"
-#include "memory.h"
-#include "bi_vars.h"
-#include "field.h"
-#include "symtype.h"
-#include "scan.h"
+#include <mawk.h>
+#include <fin.h>
+#include <memory.h>
+#include <bi_vars.h>
+#include <field.h>
+#include <symtype.h>
+#include <scan.h>
 
 #ifdef	  HAVE_FCNTL_H
 #include <fcntl.h>
@@ -281,13 +285,13 @@ FINgets(FIN * fin, size_t *len_p)
     case SEP_STR:
 	q = str_str(p,
 		    (size_t) (fin->limit - p),
-		    ((STRING *) rs_shadow.ptr)->str,
-		    match_len = ((STRING *) rs_shadow.ptr)->len);
+		    rs_shadow.u.s_ptr->str,
+		    match_len = (rs_shadow.u.s_ptr)->len);
 	break;
 
     case SEP_MLR:
     case SEP_RE:
-	q = re_pos_match(p, (size_t) (fin->limit - p), rs_shadow.ptr,
+	q = re_pos_match(p, (size_t) (fin->limit - p), rs_shadow.u.r_ptr,
 			 &match_len,
 			 (p != fin->buff) ||
 			 (fin->flags & FIN_FLAG));
@@ -376,11 +380,12 @@ enlarge_fin_buffer(FIN * fin)
     fin->buffp =
 	fin->buff = (char *) zrealloc(fin->buff, oldsize, newsize);
 
-    r = fillbuff(fin->fd, fin->buff + oldsize, extra);
-    if (r < extra)
-	fin->flags |= EOF_FLAG;
-
-    fin->limit = fin->buff + limit + r;
+    if (fin->fp == 0) {
+	r = fillbuff(fin->fd, fin->buff + oldsize, extra);
+	if (r < extra)
+	    fin->flags |= EOF_FLAG;
+	fin->limit = fin->buff + limit + r;
+    }
     return fin->buff;
 }
 
@@ -546,13 +551,16 @@ next_main(int open_flag)	/* called by open_main() if on */
 int
 is_cmdline_assign(char *s)
 {
+    static CELL empty_cell;
+
     register char *p;
+
     int c;
     SYMTAB *stp;
     CELL *cp = 0;
     size_t len;
-    CELL cell;			/* used if command line assign to pseudo field */
-    CELL *fp = (CELL *) 0;	/* ditto */
+    CELL cell = empty_cell;	/* used if command line assign to pseudo field */
+    CELL *fp = NULL;		/* ditto */
     size_t length;
 
     if (scan_code[*(unsigned char *) s] != SC_IDCHAR)
