@@ -1,6 +1,6 @@
 /********************************************
 rexp.h
-copyright 2008-2021,2023, Thomas E. Dickey
+copyright 2008-2023,2024, Thomas E. Dickey
 copyright 2010, Jonathan Nieder
 copyright 1991,2014, Michael D. Brennan
 
@@ -12,14 +12,15 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp.h,v 1.41 2023/07/28 00:21:02 tom Exp $
+ * $MawkId: rexp.h,v 1.43 2024/08/25 17:15:43 tom Exp $
  */
 
 #ifndef  REXP_H
 #define  REXP_H
 
-#include "nstd.h"
-#include "types.h"
+#include <nstd.h>
+#include <types.h>
+
 #include <stdio.h>
 #include <setjmp.h>
 
@@ -59,7 +60,9 @@ typedef enum {
 
 typedef UChar BV[32];		/* bit vector */
 
-typedef struct {
+typedef struct _state
+#ifdef Visible_STATE
+{
     SType s_type;
     size_t s_len;		/* used for M_STR  */
     union {
@@ -72,13 +75,19 @@ typedef struct {
     Int it_max;			/* used for s_type == M_2JC */
     Int it_cnt;
 #endif
-} STATE;
+}
+#endif
+STATE;
 
 #define  STATESZ  (sizeof(STATE))
 
-typedef struct {
+typedef struct _machine
+#ifdef Visible_MACHINE
+{
     STATE *start, *stop;
-} MACHINE;
+}
+#endif
+MACHINE;
 
 /*  tokens   */
 typedef enum {
@@ -124,17 +133,23 @@ typedef enum {
 #define  ison(b,x)  ((b)[((UChar)(x)) >> 3] & (1 << ((x) & 7)))
 
 /* struct for the run time stack */
-typedef struct {
+typedef struct _rt_state
+#ifdef Visible_RT_STATE
+{
     STATE *m;			/* save the machine ptr */
     int u;			/* save the u_flag */
     char *s;			/* save the active string ptr */
     int sp;			/* size of position stack */
     int tp;			/* offset to top entry of position stack */
     char *ss;			/* save the match start -- only used by REmatch */
-} RT_STATE;			/* run time state */
+}
+#endif
+RT_STATE;			/* run time state */
 
 /* entry for the position stack */
-typedef struct {
+typedef struct _rt_pos_entry
+#ifdef Visible_RT_POS_ENTRY
+{
     /* if we have not advanced beyond this character,
      * do not bother trying another round.
      */
@@ -144,7 +159,9 @@ typedef struct {
     int owner;
     /* previous node is this - this->prev_offset.  See RE_pos_pop() */
     int prev_offset;
-} RT_POS_ENTRY;
+}
+#endif
+RT_POS_ENTRY;
 
 /*  error  trap   */
 extern int REerrno;
@@ -213,18 +230,18 @@ RE_pos_push(RT_POS_ENTRY * head, const RT_STATE * owner, const char *s)
 static /* inline */ const char *
 RE_pos_pop(RT_POS_ENTRY ** head, const RT_STATE * current)
 {
-    RT_POS_ENTRY *prev = *head - (*head)->prev_offset;
+    RT_POS_ENTRY *prev2 = *head - (*head)->prev_offset;
 
-    if (prev->owner == current - RE_run_stack_base) {	/* likely */
+    if (prev2->owner == current - RE_run_stack_base) {	/* likely */
 	/* no need to preserve intervening nodes */
-	*head = prev;
-    } else if (*head == prev) {
+	*head = prev2;
+    } else if (*head == prev2) {
 	RE_panic("unbalanced M_SAVE_POS and M_2JC");
     } else {
-	(*head)->prev_offset += prev->prev_offset;
+	(*head)->prev_offset += prev2->prev_offset;
     }
 
-    return prev->pos;
+    return prev2->pos;
 }
 
 #ifndef NO_INTERVAL_EXPR
