@@ -1,6 +1,6 @@
 /********************************************
 rexp0.c
-copyright 2008-2016,2020, Thomas E. Dickey
+copyright 2008-2020,2024, Thomas E. Dickey
 copyright 2010, Jonathan Nieder
 copyright 1991-1994,1996, Michael D. Brennan
 
@@ -12,13 +12,13 @@ the GNU General Public License, version 2, 1991.
 ********************************************/
 
 /*
- * $MawkId: rexp0.c,v 1.46 2020/10/16 22:19:44 tom Exp $
+ * $MawkId: rexp0.c,v 1.48 2024/08/25 17:16:24 tom Exp $
  */
 
 /*  lexical scanner  */
 
 #undef LOCAL_REGEXP		/* no need for push/pop */
-#include  "rexp.h"
+#include  <rexp.h>
 
 #include <ctype.h>
 
@@ -369,7 +369,7 @@ RE_lex(MACHINE * mp)
 	    return prev = T_END;
 
 	default:
-	    RE_panic("bad switch in RE_lex");
+	    RE_panic("bad switch in RE_lex: %d", c);
 	}			/* T_CAT switch */
 	break;
 
